import {  } from '@preserves/core';
import { compile, Meta as M, readSchema } from '../src/index';
import './test-utils';
import { js as format } from 'js-beautify';

function compileSingleSchema(source: string): string {
    return format(compile([], [Symbol.for('test')], readSchema(`version 1 . ${source}`), {}));
}

describe('compiler', () => {
    it('basics', () => {
        expect(compileSingleSchema(`X = int .`)).toContain(`\nexport type X = number;\n`);
    });

    it('symbol-keyed dictionary', () => {
        const s = compileSingleSchema(`D = { symbol:any ...:... } .`);
        expect(s).toMatch(/^export type D.*= _.JsDictionary < _.Value < _embedded >> ;$/m);
    });

    it('string-keyed dictionary', () => {
        const s = compileSingleSchema(`D = { string:any ...:... } .`);
        expect(s).toMatch(/^export type D.*= _.EncodableDictionary < _embedded, string, _.Value < _embedded >> ;$/m);
    });

    it('any-keyed dictionary', () => {
        const s = compileSingleSchema(`D = { any:any ...:... } .`);
        expect(s).toMatch(/^export type D.*= _.EncodableDictionary < _embedded, _.Value < _embedded > , _.Value < _embedded >> ;$/m);
    });
});
